/*
? [2024] Microchip Technology Inc. and its subsidiaries.

    Subject to your compliance with these terms, you may use Microchip 
    software and any derivatives exclusively with Microchip products. 
    You are responsible for complying with 3rd party license terms  
    applicable to your use of 3rd party software (including open source  
    software) that may accompany Microchip software. SOFTWARE IS ?AS IS.? 
    NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS 
    SOFTWARE, INCLUDING ANY IMPLIED WARRANTIES OF NON-INFRINGEMENT,  
    MERCHANTABILITY, OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT 
    WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY 
    KIND WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF 
    MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE 
    FORESEEABLE. TO THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP?S 
    TOTAL LIABILITY ON ALL CLAIMS RELATED TO THE SOFTWARE WILL NOT 
    EXCEED AMOUNT OF FEES, IF ANY, YOU PAID DIRECTLY TO MICROCHIP FOR 
    THIS SOFTWARE.
*/

#include "mcc_generated_files/system/system.h"
#include "mcc_generated_files/secondary_core/sec_core1.h"
#include "mcc_generated_files/system/pins.h"
#include "mcc_generated_files/timer/tmr1.h"
#include "OLED128x64.h"
#include "mcc_generated_files/uart/uart1.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>


#define DATA_UNDER_TEST       0xAAAA       // data used for protocol write and read test
#define MCP9808_ADDR          0x1f         //Tempurature Sensor Address
#define MAX_UART_DATA_LENGTH  100

/* Create a pointer of type SEC_CORE_INTERFACE and assign it to the address of the MSIInterface SEC_CORE_INTERFACE struct.
This enables us to get access the portable API interface. */
const struct SEC_CORE_INTERFACE *secondaryCore =  &MSIInterface;
volatile unsigned char  I2C_Wbuffer[16];   
volatile unsigned char  I2C_Rbuffer[16];
unsigned int            UART_Duty[2];
unsigned int            MCP9808Value = 0 ;
unsigned char           ASCII_Buffer[20];

uint16_t dataSend[MSI1_ProtocolA_SIZE];      //for Protocol size refer sec_core_types.h file
uint16_t dataReceive[MSI1_ProtocolB_SIZE];

/*
    Main application
*/

void Main_Interrupt(void) 
{
    LED1_Toggle();
}

int main(void)
{
    SYSTEM_Initialize();
    
    SEC_CORE1_Initialize();
    SEC_CORE1_Program();
    SEC_CORE1_Start();

    
    dataSend[0] =  DATA_UNDER_TEST;         //Initializing to known value.
    dataReceive[0] = 0;
    
    //Mailbox write
    SEC_CORE1_ProtocolWrite(MSI1_ProtocolA, (uint16_t*)dataSend);
    
    //Issue interrupt to secondary
    SEC_CORE1_InterruptRequestGenerate();
    while(!SEC_CORE1_IsInterruptRequestAcknowledged());
    SEC_CORE1_InterruptRequestComplete();
    while(SEC_CORE1_IsInterruptRequestAcknowledged());
    
    //Wait for interrupt from secondary
    while(!SEC_CORE1_IsInterruptRequested());
    SEC_CORE1_InterruptRequestAcknowledge();
    while(SEC_CORE1_IsInterruptRequested());
    SEC_CORE1_InterruptRequestAcknowledgeComplete(); 
    
    //Mailbox read
    SEC_CORE1_ProtocolRead(MSI1_ProtocolB, (uint16_t*)dataReceive);
    
    //Glow LED on data match
    if(memcmp(dataSend, dataReceive, sizeof(dataSend)) == 0)
    {
        LED1_SetHigh();
    }
    else
    {
        LED1_SetLow();
    }
    
    TMR1_TimeoutCallbackRegister(Main_Interrupt);
    
    OLED_Init();
    OLED_CLS();
    
    I2C_Wbuffer[0] = 0x01 ;
    I2C_Wbuffer[1] = 0x00 ;
    I2C_Wbuffer[2] = 0x04 ;
    I2C1_Write(MCP9808_ADDR,I2C_Wbuffer,2);
    while ( I2C1_IsBusy()) ;

    I2C_Wbuffer[0] = 0x02 ;
    I2C_Wbuffer[1] = 0x05 ;
    I2C_Wbuffer[2] = 0xa0 ;
    I2C1_Write(MCP9808_ADDR,I2C_Wbuffer,3);
    while ( I2C1_IsBusy()) ;

    I2C_Wbuffer[0] = 0x03 ;
    I2C_Wbuffer[1] = 0x00 ;
    I2C_Wbuffer[2] = 0x10 ;
    I2C1_Write(MCP9808_ADDR,I2C_Wbuffer,3);
    while ( I2C1_IsBusy()) ;   

    I2C_Wbuffer[0] = 0x04 ;
    I2C_Wbuffer[1] = 0x05 ;
    I2C_Wbuffer[2] = 0xa0 ;
    I2C1_Write(MCP9808_ADDR,I2C_Wbuffer,3);
    while ( I2C1_IsBusy()) ;
    
    OLED_Put8x16Str(0,0,"dsPIC33CH Melody");
    OLED_Put8x16Str(0,4,"Temp:");
  
    while(1)
    {
        printf(" hello world\r\n");
        
        UART_Duty[0] = UART1_Read();
        UART_Duty[1] = UART1_Read();
        
        printf(" DutyCycle = %c%c\r\n", (char)UART_Duty[0], (char)UART_Duty[1]);

        dataSend[0] = (UART_Duty[0]-48);
        dataSend[1] = (UART_Duty[1]-48);

        SEC_CORE1_ProtocolWrite(MSI1_ProtocolA, (uint16_t*)dataSend);
        printf(" mailbox = %d%d\r\n", MSI1MBX0D, MSI1MBX1D);

        I2C_Wbuffer[0] = 5 ;
        I2C1_WriteRead(MCP9808_ADDR,I2C_Wbuffer,1,I2C_Rbuffer,2);
        while ( I2C1_IsBusy()) ;
        
        MCP9808Value = (unsigned int)(I2C_Rbuffer[0] * 256) + I2C_Rbuffer[1];   
        MCP9808Value  = MCP9808Value >> 4 ;
        MCP9808Value &= 0x00ff ;
        sprintf (ASCII_Buffer,"%3d",MCP9808Value) ;
        OLED_Put8x16ASCII(44,4,3,(uint8_t*)ASCII_Buffer) ;
    }    
}